package mezz.jei.common.input.handlers;

import com.google.common.base.MoreObjects;
import mezz.jei.common.input.IInternalKeyMappings;
import mezz.jei.common.input.UserInput;
import mezz.jei.common.input.IUserInputHandler;
import mezz.jei.common.util.ImmutableRect2i;
import net.minecraft.class_3675;
import net.minecraft.class_437;
import org.jetbrains.annotations.Nullable;
import java.util.Optional;

public class LimitedAreaInputHandler implements IUserInputHandler {
	private final IUserInputHandler handler;
	private final ImmutableRect2i area;

	public static IUserInputHandler create(IUserInputHandler handler, @Nullable ImmutableRect2i area) {
		if (area == null) {
			return handler;
		}
		return new LimitedAreaInputHandler(handler, area);
	}

	private LimitedAreaInputHandler(IUserInputHandler handler, ImmutableRect2i area) {
		this.handler = handler;
		this.area = area;
	}

	@Override
	public String toString() {
		return MoreObjects.toStringHelper(this)
			.add("area", this.area)
			.add("handler", this.handler)
			.toString();
	}

	@Override
	public Optional<IUserInputHandler> handleUserInput(class_437 screen, UserInput input, IInternalKeyMappings keyBindings) {
		if (this.area.contains(input.getMouseX(), input.getMouseY())) {
			return this.handler.handleUserInput(screen, input, keyBindings)
				.map(handled -> this);
		}
		return Optional.empty();
	}

	@Override
	public void handleMouseClickedOut(class_3675.class_306 input) {
		this.handler.handleMouseClickedOut(input);
	}

	@Override
	public Optional<IUserInputHandler> handleMouseScrolled(double mouseX, double mouseY, double scrollDelta) {
		if (this.area.contains(mouseX, mouseY)) {
			return this.handler.handleMouseScrolled(mouseX, mouseY, scrollDelta);
		}
		return Optional.empty();
	}
}
